from typing import Any, Literal

from pydantic import BaseModel, Field

from ptsandbox.models.core.base import BaseRequest
from ptsandbox.models.core.enum import (
    EOSStatus,
    SystemCode,
    SystemGroup,
    UploadStrategy,
)
from ptsandbox.models.ui.common import SMTPDefaultRecord


class SandboxSystemStatusResponse(BaseModel):
    """
    System events are returned here with information about the current state of the system, i.e. a slice of the current state.

    The event may relate to a specific object, in this case the ObjectId field is filled in. Basically, the event codes indicate a specific problem.
    Some events may contain additional parameters. For example `AV_ENGINE_IN_NOT_GENERAL_AVAILABILITY`

    Description of current errors:

    * `AV_ENGINE_IN_NOT_GENERAL_AVAILABILITY` - the engine is approaching or becoming unavailable. The ObjectId specifies the name of the engine.
    Example of parameters
    ```
    {
        "engineCodeName": "clamav",
        "distributionPack": "...",
        "distributionVersion": "1.1.1",
        "maintenanceStatus": "END_OF_SUPPORT"
    }
    ```

    * `STORAGE_CAPACITY_EXCEEDED_COMPONENTS_MAX_SIZE_BYTES` - Exceeding limits for a specific component

    * `STORAGE_CAPACITY_EXCEEDED_MINIMUM` - Exceeding the total minimum required free space

    * `COMPONENT_CAPACITY_EXCEEDED_THRESHOLD` - Exceeding the threshold for free space for a specific component (ObjectId)

    * `COMPONENT_CAPACITY_EXCEEDED_MAX_SIZE` - Exceeding the limit for a specific component (ObjectId)
    A list of possible components, similar to the Settings API: `['incubator', 'quarantine', 'system', 'sandbox_images']`

    * `SANDBOX_RECONFIGURING` - The configuration is being performed.

    * `NODE_IS_NOT_READY` - The node is unavailable (with the not ready status). The ObjectId contains the node name.
    The parameters indicate the time from which the status changed.
    ```
    {
        "since": 1111111.0
    }
    ```

    * `COMPONENT_ERROR` - An error in specific components of the group. The ObjectId specifies a group of components. Example:
    ```
    {
        "code": "COMPONENT_ERROR",
        "group": "SANDBOX",
        "level": "ERROR",
        "objectId": "SANDBOX",
        "params": {
            "sb-worker-1": {
                "error_codes": [
                    "OFFLINE"
                ]
            },
            "traps-resource-plugin": {
                "error_codes": [
                    "OFFLINE"
                ]
            }
        }
        ...
    }
    ```
    This error should be interpreted roughly as: An error of the "SANDBOX" subsystem. Affected components: `"sb-worker-1", "traps-resource-plugin"`

    * `COMPONENT_PARTIALLY_AVAILABLE` - Similar to COMPONENT_ERROR, only it's about the unavailability of a part of the pod, or a decrease in performance

    * `NEW_VERSION_AVAILABLE` - A new version is available. The ObjectId specifies the version

    * `NEW_VERSION_INSTALLATION_SCHEDULED` - A new version is available and it is scheduled for installation. The ObjectId specifies the version. In the settings, the scheduled installation time is
    ```
    {
        "time": 1111111
    }
    ```

    * `END_OF_SUPPORT_SOON` - Support for the `END_OF_SUPPORT` version will end soon - Support for the version has been discontinued in params
    ```
    {
        "eosTs": 1111111
    }
    ```

    * `IMAGE_INSTALL_ERROR` - Error when installing the image, the name of the image is entered in the ObjectId.

    * `NODE_HAVE_ERROR` - There are errors on the node. The ObjectId contains the node name. The error types are specified in the parameters.
    ```
    {
        "error_types": ["NetworkUnavailable", "MemoryPressure", "DiskPressure"]
    }
    ```
    """

    class Event(BaseModel):
        group: SystemGroup
        """
        Group (subsystem)
        """

        code: SystemCode
        """
        Event/Error code
        """

        object_id: str = Field(alias="objectId")
        """
        The object's ID. It can be an empty string for general events.
        """

        level: Literal["INFO", "WARNING", "ERROR"]
        """
        Event/Error level
        """

        created_ts: int = Field(alias="createdTs")
        """
        Time when the event was created
        """

        updated_ts: int = Field(alias="updatedTs")
        """
        Event update Time
        """

        params: dict[Any, Any] = {}
        """
        Additional event parameters
        """

    events: list[Event]


class SandboxSystemSettingsResponse(BaseModel):
    """
    System Settings
    """

    class Data(BaseModel):
        class Telemetry(BaseModel):
            enabled: bool
            """
            Telemetry status
            """

            send_events: bool = Field(alias="sendEvents")

        class SIEMNotifier(BaseModel):
            enabled: bool
            """
            Status of event collection and analysis
            """

            host: str
            """
            Syslog Server
            """

            port: int
            """
            Port syslog server
            """

            transport_protocol: Literal["tcp", "udp"] = Field(alias="transportProtocol")
            """
            Transmission protocol
            """

            audit_enabled: bool = Field(alias="auditEnabled")
            """
            Send audit events to the system log
            """

        class CybsiNotifier(BaseModel):
            enabled: bool
            """
            Enabling/disabling sending reports to Cybsi
            """

            api_url: str = Field(alias="apiUrl")
            """
            Cybsi URL API
            """

            api_key: str = Field(alias="apiKey")
            """
            Key for the Cybsi API
            """

            share_level: Literal["White", "Green", "Amber", "Red"] = Field(alias="shareLevel")
            """
            The access level applied to all artifacts
            """

            upload_strategy: UploadStrategy = Field(alias="uploadStrategy")
            """
            The rule for uploading artifacts
            """

        class EmailNotifier(BaseModel):
            enabled: bool
            """
            Enabling/disabling the sending of notifications by mail
            """

            notify_unwanted: bool = Field(alias="notifyUnwanted")
            """
            Notification of unwanted objects
            """

            locale: str
            """
            Locale for generated messages
            """

            sender_address: str = Field(alias="senderAddress")
            """
            Sender's address
            """

            recipients: list[str]
            """
            List of recipient addresses
            """

            smtp_default_records: list[SMTPDefaultRecord] = Field(alias="smtpDefaultRecords")
            """
            List of mail servers
            """

        class Authentication(BaseModel):
            anonymous_deny: bool = Field(alias="anonymousDeny")
            """
            Anonymous analysis is prohibited
            """

        class Unpacker(BaseModel):
            passwords: list[str]
            """
            List of passwords for unpacking password-protected archives
            """

        class EventCombiner(BaseModel):
            events_eviction_days: int = Field(alias="eventsEvictionDays")
            """
            Retention period of the verification history
            """

        class Storage(BaseModel):
            class Settings(BaseModel):
                bytes: int
                """
                Size in bytes
                """

                items: int
                """
                Number of files
                """

            class Components(BaseModel):
                class Component(BaseModel):
                    class Settings(BaseModel):
                        bytes: int
                        """
                        Size in bytes
                        """

                        items: int
                        """
                        Number of files
                        """

                    class Threshold(BaseModel):
                        bytes_percent: int = Field(alias="bytesPercent")
                        """
                        Percentage of threshold in bytes
                        """

                        items_percent: int = Field(alias="itemsPercent")
                        """
                        The percentage of the threshold in the number of files
                        """

                    current_limit: Settings = Field(alias="currentLimit")

                    current_size: Settings = Field(alias="currentSize")

                    min_size: Settings = Field(alias="minSize")

                    threshold: Threshold

                sandbox_images: Component = Field(alias="sandboxImages")

                quarantine: Component

                system: Component

                incubator: Component

            capacity: Settings

            current_size: Settings = Field(alias="currentSize")

            components: Components

        class Quarantine(BaseModel):
            retention_period: int = Field(alias="retentionPeriod")
            """
            Storage period in quarantine
            """

            use_smtp: bool = Field(alias="useSMTP")
            """
            Use a backup mail server to forward emails
            """

            smtp_default_records: list[SMTPDefaultRecord] = Field(alias="smtpDefaultRecords")
            """
            List of mail servers
            """

        class Retro(BaseModel):
            enabled: bool
            """
            The status of the retro check
            """

        telemetry: Telemetry

        siem_notifier: SIEMNotifier = Field(alias="siemNotifier")

        cybsi_notifier: CybsiNotifier = Field(alias="cybsiNotifier")

        email_notifier: EmailNotifier = Field(alias="emailNotifier")

        authentication: Authentication

        unpacker: Unpacker

        event_combiner: EventCombiner = Field(alias="eventCombiner")

        storage: Storage

        quarantine: Quarantine

        retro: Retro

    data: Data


class SandboxUpdateSystemSettingsRequest(BaseRequest):
    class SIEMNotifier(BaseModel):
        enabled: bool | None = None
        """
        Status of event collection and analysis
        """

        host: str | None = None
        """
        Syslog Server
        """

        port: int | None = None
        """
        Port syslog server
        """

        transport_protocol: Literal["tcp", "udp"] | None = Field(default=None, serialization_alias="transportProtocol")
        """
        Transmission protocol
        """

        audit_enabled: bool | None = Field(default=None, serialization_alias="auditEnabled")
        """
        Send audit events to the system log
        """

    class CybsiNotifier(BaseModel):
        enabled: bool | None = None
        """
        Enabling/disabling sending reports to Cybsi
        """

        api_url: str | None = Field(default=None, serialization_alias="apiUrl")
        """
        Cybsi URL API
        """

        api_key: str | None = Field(default=None, serialization_alias="apiKey")
        """
        Key for the Cybsi API
        """

        share_level: Literal["White", "Green", "Amber", "Red"] | None = Field(
            default=None, serialization_alias="shareLevel"
        )
        """
        The access level applied to all artifacts
        """

        upload_strategy: UploadStrategy | None = Field(default=None, serialization_alias="uploadStrategy")
        """
        The rule for uploading artifacts
        """

    class EmailNotifier(BaseModel):
        enabled: bool
        """
        Enabling/disabling the sending of notifications by mail
        """

        notify_unwanted: bool = Field(serialization_alias="notifyUnwanted")
        """
        Notification of unwanted objects
        """

        locale: str
        """
        Locale for generated messages
        """

        sender_address: str = Field(serialization_alias="senderAddress")
        """
        Sender's address
        """

        recipients: list[str]
        """
        List of recipient addresses
        """

        smtp_default_records: list[SMTPDefaultRecord] = Field(serialization_alias="smtpDefaultRecords")
        """
        List of mail servers
        """

    class Authentication(BaseModel):
        anonymous_deny: bool = Field(serialization_alias="anonymousDeny")
        """
        Anonymous analysis is prohibited
        """

    class Unpacker(BaseModel):
        passwords: list[str]
        """
        List of passwords for unpacking password-protected archives
        """

    class EventCombiner(BaseModel):
        events_eviction_days: int = Field(serialization_alias="eventsEvictionDays")
        """
        Retention period of the verification history
        """

    class Storage(BaseModel):
        class Components(BaseModel):
            class Component(BaseModel):
                class Settings(BaseModel):
                    bytes: int | None = None
                    """
                    Size in bytes
                    """

                    items: int | None = None
                    """
                    Number of files
                    """

                current_limit: Settings = Field(serialization_alias="currentLimit")

            quarantine: Component | None = None

            incubator: Component | None = None

        components: Components

    class Quarantine(BaseModel):
        retention_period: int | None = Field(default=None, serialization_alias="retentionPeriod")
        """
        Storage period in quarantine
        """

        use_smtp: bool | None = Field(default=None, serialization_alias="useSMTP")
        """
        Use a backup mail server to forward emails
        """

        smtp_default_records: list[SMTPDefaultRecord] | None = Field(
            default=None, serialization_alias="smtpDefaultRecords"
        )
        """
        List of mail servers
        """

    class Retro(BaseModel):
        enabled: bool
        """
        The status of the retro check
        """

    authentication: Authentication | None = None

    event_combiner: EventCombiner | None = Field(default=None, serialization_alias="eventCombiner")

    quarantine: Quarantine | None = None

    retro: Retro | None = None

    unpacker: Unpacker | None = None

    siem_notifier: SIEMNotifier | None = Field(default=None, serialization_alias="siemNotifier")

    cybsi_notifier: CybsiNotifier | None = Field(default=None, serialization_alias="cybsiNotifier")

    storage: Storage | None = None

    email_notifier: EmailNotifier | None = Field(default=None, serialization_alias="emailNotifier")


class SandboxSystemVersionResponse(BaseModel):
    class Data(BaseModel):
        version: str

        eos_ts: int = Field(alias="eosTs")

        eos_status: EOSStatus = Field(alias="eosStatus")

    data: Data



from enum import Enum
from functools import cached_property

from pydantic import BaseModel, ConfigDict, Field


class SandboxKey(BaseModel):
    """
    Abstraction over the key that is used to send to the sandbox
    """

    class UI(BaseModel):
        model_config = ConfigDict(use_enum_values=True)

        class AuthType(int, Enum):
            default = 0
            ldap = 1

        login: str
        password: str
        auth_type: AuthType = AuthType.default

    name: str
    """
    Custom key name
    """

    key: str
    """
    The key received in the sandbox interface
    """

    host: str
    """
    Hostname of the sandbox instance

    For example: `1.1.1.1` or `sandbox.example.com` without https etc
    """

    description: str = ""
    """
    A description of the key for easy representation somewhere in the interface
    """

    max_workers: int = Field(default=8, ge=1)
    """
    The maximum number of simultaneously running behavioral nodes

    The quantity can be found in the interface
    """

    ui: UI | None = None
    """
    If necessary, you can also access the sandbox via the UI API
    """

    @cached_property
    def url(self) -> str:
        """
        https address for connecting via API
        """

        return f"https://{self.host}/api/v1"

    @cached_property
    def debug_url(self) -> str:
        """
        https address for connecting via debug API
        """

        return f"https://{self.host}/api/debug"

    @cached_property
    def ui_url(self) -> str:
        """
        https address for connecting via UI API
        """

        return f"https://{self.host}/api/ui"

    def __repr__(self) -> str:
        return f"{self.name} for {self.host} ({self.max_workers})" + (
            f"({self.description})" if self.description else ""
        )

    def __key(self) -> tuple[str, str, str]:
        return (self.name, self.key, self.host)

    def __hash__(self) -> int:
        return hash(self.__key())

    def __eq__(self, value: object) -> bool:
        if isinstance(value, SandboxKey):
            return self.__key() == value.__key()

        return False



from datetime import datetime

from pydantic import BaseModel, Field

from ptsandbox.models.core import (
    BlacklistStatus,
    BootkitmonStage,
    ContextType,
    EngineName,
    ErrorType,
    FileInfoProperties,
    FileInfoTypes,
    HashType,
    HTTPDirection,
    NetworkObjectType,
    SandboxImageInfo,
    ScanArtifactType,
    ScanState,
    SuspiciousBehaviors,
    TreeEngineName,
    TreeNodeType,
)
from ptsandbox.models.ui.common import CorrelationInfo, DetectionUI, Scan


class ScanArtifact(BaseModel):
    name: str
    sha256: str
    size: int
    type: ScanArtifactType | None = None


class SandboxInfo(BaseModel):
    class MSDNError(BaseModel):
        name: str
        """
        MSDN name of the image initialization error
        """

        code: int
        """
        The MSDN number of the image initialization error
        """

    class SandboxError(BaseModel):
        type: ErrorType

        duration: int | None = None
        """
        Waiting time
        """

    class Network(BaseModel):
        database_time: int | None = Field(default=None, alias="databaseTime")

        database_version: str = Field(alias="databaseVersion")

        version: str

        detections: list[DetectionUI] = []

        suspicious_behaviors: list[SuspiciousBehaviors] = Field(alias="suspiciousBehaviors")

    analysis_duration: int = Field(alias="analysisDuration")
    """
    Duration of behavioral analysis
    """

    analysis_planned_duration: int = Field(alias="analysisPlannedDuration")
    """
    Planned duration of the analysis
    """

    dpi_rules_version: str = Field(alias="dpiRulesVersion")
    """
    PT DPI rules version
    """

    correlation_rules_version: str = Field(alias="correlationRulesVersion")
    """
    Version of the correlation rules
    """

    mitm: bool
    """
    Was MITM enabled during the scan?
    """

    file_type: str | None = Field(default=None, alias="fileType")
    """
    What type of file was launched with

    File type (apparently, how the sandbox decided)
    """

    image_info: SandboxImageInfo = Field(alias="imageInfo")
    """
    Image Information
    """

    auto_select: bool = Field(alias="autoSelect")
    """
    Was the image selected automatically
    """

    suspicious_behaviors: list[SuspiciousBehaviors] = Field(alias="suspiciousBehaviors")
    """
    List of suspicious rules
    """

    detections: list[DetectionUI]
    """
    List of malware rules
    """

    init_msdn_error: MSDNError | None = Field(default=None, alias="initMsdnError")

    errors: list[SandboxError]

    bootkitmon: bool
    """
    Was bootkitmon enabled during the scan?
    """

    bootkitmon_stage: BootkitmonStage | None = Field(default=None, alias="bootkitmonStage")
    """
    Type of bootkitmon analysis stage
    """

    stage_index: int = Field(alias="stageIndex")
    """
    The number of the bootkitmon analysis stage
    """

    network: Network | None = None

    result: CorrelationInfo | None = None


class TreeNode(BaseModel):
    class ArchiveInfo(BaseModel):
        password: str

    class Info(BaseModel):
        type: FileInfoTypes

        name: str
        size: int
        sha1: str
        sha256: str
        md5: str
        ssdeep: str
        mime_type: str = Field(alias="mimeType")
        magic_string: str = Field(alias="magicString")
        file_type: str = Field(alias="fileType")
        properties: list[FileInfoProperties]

    class EmailInfo(BaseModel):
        subject: str
        from_: str | None = Field(default=None, alias="from")
        to: list[str]
        cc: list[str]
        bcc: list[str]

    class UrlInfo(BaseModel):
        class Redirect(BaseModel):
            url: str | None = None
            status: int | None = None

        url: str | None = None
        redirects: list[Redirect] | None = None

    class SandboxDropInfo(BaseModel):
        process_id: int = Field(alias="processId")
        process_name: str = Field(alias="processName")
        create_time: int = Field(alias="createTime")
        trigger: str
        bootkitmon: bool
        """
        Was bootkitmon enabled during the scan?
        """

        bootkitmon_stage: BootkitmonStage = Field(alias="bootkitmonStage")
        """
        ID of the bootkitmon analysis stage
        """

        stage_index: int | None = Field(alias="stageIndex")
        """
        The number of the bootkitmon analysis stage
        """

        graph_node_id: int | None = Field(alias="graphNodeId")
        """
        ID of the node in the BA graph
        """

    class SandboxCorrelatedInfo(BaseModel):
        result: CorrelationInfo

    class HTTPInfo(BaseModel):
        class Request(BaseModel):
            method: str
            url: str
            host: str

            user_agent: str = Field(alias="userAgent")
            """
            The value of the HTTP header 'User-Agent'
            """

            x_forwarded_for: str = Field(..., alias="xForwardedFor")
            """
            The value of the HTTP header 'X-Forwarded-For' is used to determine the IP of the HTTP client
            """

            referer: str
            """
            The value of the HTTP 'Referer' header, from which page the request was sent
            """

            content_type: str = Field(alias="contentType")

        class Response(BaseModel):
            code: int
            reason: str
            server: str
            content_type: str = Field(alias="contentType")
            content_disposition: str = Field(alias="contentDisposition")

        direction: HTTPDirection
        """
        Direction of HTTP query
        """

        request: Request | None = None

        response: Response | None = None

    class UnpackerInfo(BaseModel):
        class Error(BaseModel):
            type: ErrorType

            duration: int | None = None
            """
            Waiting time
            """

            limit_size: int | None = Field(None, alias="limitSize")
            """
            The value of the restriction
            """

        state: ScanState
        """
        Unpacking status
        """

        errors: list[Error] = []

    class DownloadUrlInfo(BaseModel):
        class Error(BaseModel):
            type: ErrorType

            duration: int | None = None
            """
            Waiting time
            """

            limit_size: int | None = Field(None, alias="limitSize")
            """
            The value of the restriction
            """

        state: ScanState
        """
        Url loading status
        """

        errors: list[Error] = []

        version: str
        """
        Engine version
        """

        status_code: int = Field(alias="statusCode")
        """
        Status code from the HTTP Status Line
        """

        reason_phrase: str = Field(alias="reasonPhrase")
        """
        The reason for the code from the HTTP Status Line
        """

    class BwListsInfo(BaseModel):
        class Error(BaseModel):
            type: ErrorType

            duration: int | None = None
            """
            Waiting time
            """

        state: ScanState
        """
        Check status in WB lists
        """

        status: BlacklistStatus
        """
        The result of the check on the WB lists
        """

        hashes: list[HashType]
        """
        The type of hash for which a match was found in the WB lists
        """

        errors: list[Error] = []
        """
        Errors in checking by WB lists
        """

    class CategorizerInfo(BaseModel):
        class Engine(BaseModel):
            class Error(BaseModel):
                type: ErrorType

                duration: int | None = None
                """
                Waiting time
                """

            name: TreeEngineName

            database_time: datetime | None = Field(default=None, alias="databaseTime")

            version: str

            errors: list[Error] = []

        class Result(BaseModel):
            class Error(BaseModel):
                type: ErrorType

                duration: int | None = None
                """
                Waiting time
                """

            state: ScanState
            """
            Check status in PTCategorizer
            """

            categories: list[str] = []
            """
            PTCategorizer Categories
            """

            errors: list[Error] = []
            """
            PTCategorizer check errors
            """

        engine: Engine

        result: Result | None = None

    class CacheInfo(BaseModel):
        source_scan_id: str = Field(alias="sourceScanId")
        """
        The original task ID
        """

        source_node_id: str = Field(alias="sourceNodeId")
        """
        The source node ID
        """

        timestamp: int
        """
        The time of creation of the initial task (UNIX timestamp)
        """

    class NetworkObject(BaseModel):
        type: NetworkObjectType
        """
        Type of network object
        """

        value: str
        """
        The value of the network object
        """

        is_scanned: bool = Field(alias="isScanned")
        """
        Has the network artifact been scanned?
        """

    class ParentObjectInfo(BaseModel):
        type: FileInfoTypes
        """
        The type of artifact that the current node was derived from.
        """

        name: str
        """
        The name of the artifact from which the current node was derived.
        """

    class ContextCrawlerInfo(BaseModel):
        url: str
        """
        The URL from which the file was received
        """

        engine_name: EngineName | None = None
        """
        The name of the engine used for downloading
        """

    node_id: int = Field(alias="nodeId")
    """
    The node ID. It starts from 1
    """

    parent_ids: list[int] | None = Field(None, alias="parentIds")
    """
    A list of parent node IDs. It starts from the root

    The chain! parents. 0 -> 1 -> 2 -> 3 <=> nodeId=3, parentIds=[0,1,2]
    """

    node_type: TreeNodeType = Field(..., alias="nodeType")
    """
    Node Type
    """

    scans: list[Scan]
    """
    List of scans
    """

    info: Info
    """
    Node information - hashes, file name, mime type
    """

    correlation: CorrelationInfo | None = None
    """
    Correlation results
    """

    rule_engine_info: Scan | None = Field(default=None, alias="ruleEngineInfo")

    email_info: EmailInfo | None = Field(default=None, alias="emailInfo")

    archive_info: ArchiveInfo | None = Field(default=None, alias="archiveInfo")
    """
    If the node is an archive, and the sandbox has managed to find a password, it will be in this field.
    """

    url_info: UrlInfo | None = Field(default=None, alias="urlInfo")

    sandbox_info: SandboxInfo | None = Field(default=None, alias="sandboxInfo")
    """
    Sandbox-specific scan results
    """

    sandbox_drop_info: SandboxDropInfo | None = Field(default=None, alias="sandboxDropInfo")
    sandbox_procdump_info: SandboxDropInfo | None = Field(default=None, alias="sandboxProcDumpInfo")
    sandbox_memdump_info: SandboxDropInfo | None = Field(default=None, alias="sandboxMemDumpInfo")

    sandbox_correlated_info: SandboxCorrelatedInfo | None = Field(default=None, alias="sandboxCorrelatedInfo")
    """
    Correlated sandbox result (filled in if sandbox_correlated_state != UNKNOWN)
    """

    scan_artifacts: list[ScanArtifact] | None = Field(default=None, alias="scanArtifacts")
    """
    Sandbox artifacts: trails, events, graph, videos...
    """

    http_info: HTTPInfo | None = Field(default=None, alias="httpInfo")

    unpacker_info: UnpackerInfo | None = Field(default=None, alias="unpackerInfo")

    download_url_info: DownloadUrlInfo | None = Field(default=None, alias="downloadUrlInfo")

    bw_lists_info: BwListsInfo | None = Field(default=None, alias="bwListsInfo")

    ptcategorizer_info: CategorizerInfo | None = Field(default=None, alias="ptcategorizerInfo")

    yara_test_info: Scan | None = Field(default=None, alias="yaraTestInfo")

    yara_main_info: Scan | None = Field(default=None, alias="yaraMainInfo")

    cache_info: CacheInfo | None = Field(default=None, alias="cacheInfo")

    network_objects: list[NetworkObject] | None = Field(default=None, alias="networkObjects")

    parent_object_info: ParentObjectInfo | None = Field(default=None, alias="parentObjectInfo")

    context_type: ContextType | None = Field(default=None, alias="contextType")

    context_crawler_info: ContextCrawlerInfo | None = Field(default=None, alias="contextCrawlerInfo")

    first_child_count: int = Field(alias="firstChildCount")
    """
    Number of children of the first level
    """

    is_match: bool | None = Field(default=None, alias="isMatch")
    """
    Used for the filtering API.

    Does the node match the search conditions, if false, then it's just the parent element?
    """

    matched_fields: list[list[str]] | None = Field(default=None, alias="matchedFields")
    """
    Used for the filtering API. The list of fields that fall under the text query

    The path to the field, for example ['info', 'name']
    """


class SandboxTreeResponse(BaseModel):
    children: list[TreeNode]

    has_more: bool = Field(default=False, alias="hasMore")
    """
    If true, the number of records is greater than the limit and you can get additional ones using 'offset'
    """
